

from mlconstants import TARGET

# Dropped by the MX Reading
readmx_drop_columns_arr = ['BidOpen', 'BidHigh', 'BidLow', 'BidClose', 'AskOpen', 'AskHigh','AskLow', 'AskClose', 'fh13', 'fl13', 'fh21', 'fl21', 'fh34', 'fl34', 'fh55','fl55','price_peak_above', 'price_peak_bellow', 'ao_peak_above','ao_peak_bellow'] #@STCissue Could :'price_peak_above', 'price_peak_bellow', 'ao_peak_above','ao_peak_bellow' be usefull features ??

mx_common_columns=[TARGET, 'vaoc','fdb']

columnsToDrop_part01_2407 = ['vaos','vaob','vaosc','vaobc','fh8', 'fl8', 'fh89', 'fl89','mfi','aoaz','aobz','sz','bz','acb','acs','ss','sb','mfi_sq', 'mfi_green','mfi_fade', 'mfi_fake','tmax', 'tmin', 'p', 'l']





import pandas as pd
from sklearn.linear_model import LinearRegression
from sklearn.metrics import mean_squared_error

import os

class Config:
  def __init__(self, instrument='SPX500', timeframe='D1', force_regenerate_mxfiles=True, mfi_flag=True, balligator_flag=True, talligator_flag=True, regenerate_cds=True, use_fresh=True, quiet=False, jgtdroot_default="/b/Dropbox/jgt", drop_subdir="drop", result_file_basename_default="jgtml_obsds_240515_TIDE_SIGNALS.result"):
    self.instrument = instrument
    self.timeframe = timeframe
    self.force_regenerate_mxfiles = force_regenerate_mxfiles
    self.mfi_flag = mfi_flag
    self.balligator_flag = balligator_flag
    self.talligator_flag = talligator_flag
    self.regenerate_cds = regenerate_cds
    self.use_fresh = use_fresh
    self.quiet = quiet
    self.jgtdroot = os.getenv("jgtdroot", jgtdroot_default)
    self.drop_subdir = drop_subdir
    self.result_file_basename = result_file_basename_default

  def get_config(self):
    return {
      'instrument': self.instrument,
      'timeframe': self.timeframe,
      'force_regenerate_mxfiles': self.force_regenerate_mxfiles,
      'mfi_flag': self.mfi_flag,
      'balligator_flag': self.balligator_flag,
      'talligator_flag': self.talligator_flag,
      'regenerate_cds': self.regenerate_cds,
      'use_fresh': self.use_fresh,
      'quiet': self.quiet,
      'jgtdroot': self.jgtdroot,
      'drop_subdir': self.drop_subdir,
      'result_file_basename': self.result_file_basename
    }

class TideAlligatorAnalysis:
  def __init__(self, instrument, timeframe):
    self.instrument = instrument
    self.timeframe = timeframe

  def read_market_data(self):
    # Implement functionality to read historical market data
    pass

  def calculate_indicators(self):
    # Implement functionality to calculate technical indicators
    pass

  def identify_signals(self):
    # Implement functionality to identify valid SELL and BUY signals
    pass

  def evaluate_profitability(self):
    # Implement functionality to evaluate the profitability of each valid signal's direction
    pass

  def analyze_profit_distribution(self):
    # Implement functionality to analyze the distribution of profit potential across different signal types
    pass

  def compare_performance(self):
    # Implement functionality to compare the performance of different signal types
    pass

  def output_results(self):
    # Implement functionality to output the results of the analysis as a CSV file
    pass

  def generate_summary(self):
    # Implement functionality to generate a markdown file summarizing the key findings
    pass




TTF_DEFAULT_PATTERN_NAME = "ttf"

TTFCLI_DESCRIPTION = "Create ttf pattern."
TTFCLI_PROG_NAME = "ttfcli"
TTFCLI_EPILOG = "This is probably an intermediary dataset creator for a pattern that are used in the MLF and other modules and it might change over time as the pattern evolves."


MLFCLI_DESCRIPTION = "Create MLF Data (alpha)"
MLFCLI_EPILOG="MLF Is used to create lagging features.  It uses the TTF data and creates lagging features for the given instrument and timeframe.  If a pattern is given and dont exist, it will try to create it using TTF."
MLFCLI_PROG_NAME="mlfcli"

PNCLI_DESCRIPTION = "Create or read pattern columns."
PNCLI_PROG_NAME = "pncli"
PNCLI_EPILOG = "This tool is used to create patterns with their corresponding columns or read existing patterns from disk."


MXCLI_PROG_NAME = "mxcli"
MXCLI_DESCRIPTION = "Create or read mx."
MXCLI_EPILOG = "This tool is used to create mx data with the given instrument and timeframe or read existing mx data from disk.  (Since the adding on Pattern, the mx data generation will change and evolve over time as we might need to add more features to load MX and the Desired Pattern then concat/merge the datasets (Efficiency))."




"""
Common functions for various cli of the jgtml project

- instrument/timeframe arguments
- force_refresh
- columns_list_from_higher_tf
- bars_amount_V2_arguments
- use_fresh_argument
- dropna_volume_argument
- drop_bid_ask
- columns_to_keep
- columns_to_drop
- lag_period
- total_lagging_periods
- patternname

"""


import argparse
from jgtutils import jgtcommon
from jgtutils.jgtcommon import _get_group_by_title


from mlcliconstants import TTF_DEFAULT_PATTERN_NAME
from jgtutils.jgtcommon import add_patterns_arguments,add_timeframe_standalone_argument,new_parser,add_format_outputs_arguments
  
def __deprecate_force_read(args:argparse.Namespace):
  try:
    if hasattr(args,'force_read') and args.force_read is True:
      print("force_read is deprecated.  Use --fresh instead")
  except:
    pass

# def add_format_outputs_arguments(parser:argparse.ArgumentParser=None)->argparse.ArgumentParser:
#   global default_parser
#   if parser is None:
#     parser=default_parser
  
#   out_group=_get_group_by_title(parser,"Outputs")
#   f_exclusive=out_group.add_mutually_exclusive_group()
#   f_exclusive.add_argument("-json", "--json_output", help="Output in JSON format", action="store_true")
#   #Markdown
#   f_exclusive.add_argument("-md", "--markdown_output", help="Output in Markdown format", action="store_true")
#   return parser

# def add_patterns_arguments(parser:argparse.ArgumentParser=None)->argparse.ArgumentParser:
#   global default_parser
#   if parser is None:
#     parser=default_parser
  
#   pn_group=_get_group_by_title(parser,"Patterns")
#   pn_group.add_argument("-clh", "--columns_list_from_higher_tf", nargs='+', help="List of columns to get from higher TF.  Default is mfi_sig,zone_sig,ao", default=None)
  
#   pn_group.add_argument("-pn", "--patternname", help="Pattern Name")
  
#   pn_group.add_argument("-pls", "--list-patterns", help="List Patterns", action="store_true")
  
#   #Add the format outputs
#   parser=add_format_outputs_arguments(parser)
#   return parser

def check_arguments(args:argparse.Namespace)->argparse.Namespace:
  
  __deprecate_force_read(args)
  
  #If we are listing patterns, then we do that, print results and exit
  cli_output_patterns_list_then_exit(args)
  args=set_columns_list_from_pattern(args)
  args=_check_pattern_arguments(args)
  return args

def _check_pattern_arguments(args:argparse.Namespace,quiet=True)->argparse.Namespace:
  
  # if args.list_patterns:
  #   if not quiet:
  #     print("List Patterns")
  #   return args
  if not args.flag_columns_were_read:
    args=_convert_clh_args_as_csv_to_list(args)
  #if the patternname is not the default, then columns_list_from_higher_tf should be set
  try:
    pn = args.patternname
    columns_list = pndata__read_new_pattern_columns_list(pn=pn,args=args)
  except:
    columns_list = None
  if columns_list is None and not args.columns_list_from_higher_tf:
    example_cli = f"[...] -pn {args.patternname} -clh <col1 col2 col3>"
    print(example_cli)
    raise Exception("Pattern Name is not the default.  Please set the columns_list_from_higher_tf.  \n" + example_cli)
  # We might have passed the columns_list_from_higher_tf with CSV rathen than space separated, so we convert it to list
  return args

def cli_output_patterns_list_then_exit(args:argparse.Namespace):
    if args.list_patterns:
        output_type="object"
        try:
          json_output = True if args.json_output  else False
          md_output = True if args.markdown_output else False
          output_type="json" if json_output else "md" if md_output else "object"
        except:
          pass
        patterns_list_results=pndata__get_all_patterns(output_type=output_type,args=args)
        print(patterns_list_results)
        exit(0)



from mldatahelper import pndata__get_all_patterns, pndata__read_new_pattern_columns_list
def set_columns_list_from_pattern(args:argparse.Namespace):
  args.flag_columns_were_read=False
  if args.patternname and not args.columns_list_from_higher_tf:
    args.columns_list_from_higher_tf = pndata__read_new_pattern_columns_list(pn=args.patternname,args=args)
    args.flag_columns_were_read=True
  return args

def _convert_clh_args_as_csv_to_list(args:argparse.Namespace):
  try:
    our_first_value_with_might_have_csv:str = args.columns_list_from_higher_tf[0]
    flag_columns_have_csv_detected_string=our_first_value_with_might_have_csv.find(",")>=0
    flag_columns_are_csv_string = isinstance(our_first_value_with_might_have_csv,str) and flag_columns_have_csv_detected_string
    if args.patternname  and flag_columns_are_csv_string:
      our_new_list_items:list[str] = our_first_value_with_might_have_csv.split(",")
      c=0
      args.columns_list_from_higher_tf[0]=our_new_list_items[0]
      
      for item in our_new_list_items:      
        if c>0:
            args.columns_list_from_higher_tf.append(item)
        c+=1
  except:
    pass
  return args
        
        
        

## THAT BELLOW ARE TEST, DONT USE IT or MAKE IT WORKS.  
# #@STCIssue Mastery parent parsing
def create_parent_jgtcommon_parser(description:str,prog:str,epilog:str)->argparse.ArgumentParser:
  parser=argparse.ArgumentParser(add_help=False)
    #jgtcommon.new_parser(description,prog,epilog)
  parser.add_help=False
  parser=jgtcommon.add_instrument_timeframe_arguments(parser)
  parser=jgtcommon.add_use_fresh_argument(parser)
  parser=jgtcommon.add_bars_amount_V2_arguments(parser)
  return parser

def new_child_parser(description:str,prog:str,epilog:str)->argparse.ArgumentParser:
  parent_parser:argparse.ArgumentParser=create_parent_jgtcommon_parser(description,prog,epilog)
  parser = argparse.ArgumentParser(description=description,prog=prog+"-child",epilog=epilog)
  #parser.add_subparsers(dest='command',type[])
  
  # parser=jgtcommon.add_instrument_timeframe_arguments(parser)
  # parser=jgtcommon.add_use_fresh_argument(parser)
  # parser=jgtcommon.add_bars_amount_V2_arguments(parser)
  return parser



def parse_args(parser:argparse.ArgumentParser)->argparse.Namespace:
  #args:argparse.Namespace=jgtcommon.parse_args(parser)
  args=parser.parse_args()
  #raise "Not Implemented Fully.  #@STCIssue Parent Parser not implemented and understood"
  return args






#%%


#from jgtpy.JGTIDS import _ids_add_fdb_column_logics_v2
#from jgtpy import jgtapyhelper as th


#from jgtpy.JGTIDSSvc import get_ids
#from jgtpy.JGTIDSRequest import JGTIDSRequest
i="GBP/USD"
t="H1"

from jgtapp import ids

ids(i,t)

from jgtpy.JGTIDS
#df = get_ids(rq=JGTIDSRequest(i,t))
# %%



import json
import os

def create_entry_signal_summary(data):
    out = f"""
# Entry Signal Details
## Trade Information
- **Entry Rate:** {data['entry']}
- **Stop Rate:** {data['stop']}
- **Buy/Sell:** {data['bs']}
- **Lots:** {data['lots']}
- **Tlid ID:** {data['tlid_id']}
- **Instrument:** {data['i']}
- **Timeframe:** {data['t']}
"""

    return out

TEMPLATE_CHART_N_LINKS="""
![](charts/signal.png)
[M1](charts/M1.png)-[W1](charts/W1.png)-[D1](charts/D1.png)-[H4](charts/H4.png)-[H1](charts/H1.png)-[m15](charts/m15.png)-[m5](charts/m5.png)
"""
 


TEMPLATE_UTILITES = """
----
## Utilities


* [entry.sh](.jgt/entry.sh)
* [cancel.sh](.jgt/cancel.sh)
* [watch.sh](.jgt/watch.sh)
* [status.sh](.jgt/status.sh)
* [update.sh](.jgt/update.sh)
* [env.sh](.jgt/env.sh)
* Other scripts might include: .jgt/mv.sh, .jgt/rmtrade.sh, .jgt/xtrail.sh, .jgt/xfdb.sh

### CLI Commands

```sh
#.jgt/env.sh
#fxtr -id $OrderID $demo_arg
#fxrmorder -id $OrderID $demo_arg
#fxclosetrade -tid $trade_id $demo_arg
#fxtr -id $OrderID $demo_arg
#jgtapp fxwatchorder -id $OrderID  -d $bs \$demo_arg
#jgtapp fxmvstop -tid $trade_id -x $1 $demo_arg
#jgtapp fxrmtrade -tid $trade_id  $demo_arg
#jgtapp fxmvstopgator -tid $trade_id -i $instrument -t $timeframe --lips $demo_arg
#jgtapp fxmvstopfdb -tid $trade_id -i $instrument -t $timeframe  $demo_arg
#jgtapp fxstatusorder -id $OrderID  $demo_arg
```

#### More

* run 

```sh
#mkdir -p helps
#jgtapp --help > helps/jgtapp.txt
#fxtr --help > helps/fxtr.txt
```
### --@STCIssue Future Enhancements
* CLI Commands to run, not hard coded scripts
* Example : _fxtrupdate, _jgtsession_mksg, _jgtsession_vswsopen, _jgtsession_mkads_ctx_timeframe, _jgtsession_mkads_all_timeframes

"""

def create_bar_to_markdown(bar,title):
    mddata=f"""

## {title}
| Metric           | Value         |
|------------------|---------------|
"""
    for key, value in bar.items():
        mddata+=f"| {key} | {value} |\n"
    return mddata
        
def get_chart_n_links():
    if 'TEMPLATE_CHART_N_LINKS' in os.environ:
        return os.getenv('TEMPLATE_CHART_N_LINKS')
    return TEMPLATE_CHART_N_LINKS

def get_utilities():
    if 'TEMPLATE_UTILITES' in os.environ:
        return os.getenv('TEMPLATE_UTILITES')
    return TEMPLATE_UTILITES

def generate_markdown_from_json_file(json_filepath):
    with open(json_filepath, 'r') as file:
        data = json.load(file)
    
    signalbar = data['signalbar']
    currentbar = data['currentbar']
    
    markdown_output = create_entry_signal_summary(data)

    markdown_output += get_chart_n_links()


    markdown_output += TEMPLATE_UTILITES

    markdown_output += create_bar_to_markdown(signalbar,"Signal Bar Data")
    
    markdown_output += create_bar_to_markdown(currentbar,"Current Bar Data")


    return markdown_output

def serialize_signal_to_markdown_file_from_json_file(json_filepath,quiet=True):
    markdown_output = generate_markdown_from_json_file(json_filepath)
    markdown_filepath = json_filepath.replace('.json', '.md')
    try:  
      with open(markdown_filepath, 'w') as file:
          file.write(markdown_output)
          if not quiet:print(f">Markdown file saved to: {markdown_filepath}")
      return markdown_filepath
    except Exception as e:
      print(f"Error saving markdown file to {markdown_filepath} {e}")
      return None
  
FDBSCAN_DIR_DEFAULT="data/signals"

def get_fdbscan_dir():
    if 'FDBSCAN_DIR' in os.environ:
        return os.getenv('FDBSCAN_DIR')
    return FDBSCAN_DIR_DEFAULT
    
def serialize_signal_to_json_file(i,t,o,signal_bar,current_bar,quiet=True,signal_dir=None,ext="json",indent=2):
      if signal_dir is None:
          signal_dir=get_fdbscan_dir()
      tlid_id=o["tlid_id"]
      o["i"]=i
      o["t"]=t
      o["signalbar"]=signal_bar.to_dict()
      o["currentbar"]=current_bar.to_dict()
      
      signal_fn = create_signal_filename(i,t, tlid_id, ext)
      signal_savepath =os.path.join(signal_dir,signal_fn)
      try:      
        os.makedirs(signal_dir,exist_ok=True)
        json.dump(o,open(signal_savepath,"w"),indent=indent)
        if not quiet:print(f">Signal saved to :{signal_savepath}")
        return signal_savepath
      except Exception as e:
        print(f"Error saving signal to {signal_savepath} {e}")
        return None

def create_signal_savepath(i,t, tlid_id, ext, signal_dir=None):
    if signal_dir is None:
        signal_dir=get_fdbscan_dir()
    signal_fn = create_signal_filename(i,t, tlid_id, ext)
    signal_savepath =os.path.join(signal_dir,signal_fn)
    return signal_savepath

def create_signal_filename(i,t, tlid_id, ext):
    ifn=i.replace("/","-")
    filename = f"{ifn}_{t}_{tlid_id}.{ext}"
    return filename



import os

import pandas as pd
pd.options.mode.copy_on_write = True
from jgtutils.jgtconstants import VOLUME
# import warnings
# warnings.filterwarnings("ignore", category=FutureWarning)
# #SettingWithCopyWarning
# warnings.filterwarnings("ignore", category=pd.errors.PerformanceWarning)
# warnings.filterwarnings("ignore", category=pd.errors.SettingWithCopyWarning)

def drop_columns_if_exists(df:pd.DataFrame, columns_to_drop=None, inplace=True)->pd.DataFrame:
  if columns_to_drop is not None:
    if not inplace:
      df = df.copy()
    for col in columns_to_drop:
      if col in df.columns:
        df.drop(columns=[col], inplace=True)
  return df

def dropna_volume_in_dataframe(df:pd.DataFrame, volume_colname="", inplace=True)->pd.DataFrame:
  if volume_colname == "":
    volume_colname = VOLUME
  if not inplace:
    df = df.copy()
  if volume_colname in df.columns:
    df = df[df[volume_colname] != 0]
  return df


def convert_col_to_int(df:pd.DataFrame, colname, inplace=True)->pd.DataFrame:
  if not inplace:
    df = df.copy()
  if colname in df.columns:
    df[colname] = df[colname].astype(int)
  return df


def get_basedir(use_full:bool,ns:str)->str:
    if use_full:
        bd=os.getenv("JGTPY_DATA_FULL")
        if bd is None:
            raise Exception("JGTPY_DATA_FULL environment variable is not set.")
    else:
        bd=os.getenv("JGTPY_DATA")
        if bd is None:
            raise Exception("JGTPY_DATA environment variable is not set.")
    fulldir=os.path.join(bd,ns)
    #mkdir -p fulldir
    os.makedirs(fulldir, exist_ok=True)
    return fulldir

def get_list_of_files_in_ns(use_full:bool,ns:str)->list[str]:
  basedir=get_basedir(use_full,ns)
  files = [f for f in os.listdir(basedir) if os.path.isfile(os.path.join(basedir, f))]
  return files

def get_outfile_fullpath(i:str,t:str,use_full:bool,ns:str,pn:str="",suffix:str="",ext="csv"):
  save_basedir=get_basedir(use_full,ns)
  ifn=i.replace("/","-") if i is not None else "--"
  output_filename = f"{ifn}_{t}_{pn}{suffix}.{ext}"
  if i=="-" and t=="-":
    output_filename = f"{pn}{suffix}.{ext}"
  result_path = os.path.join(save_basedir,output_filename.replace("__","_").replace("_.",".")).replace('"','')
  return result_path